@@ -11,19 +11,25 @@
 #' @param index name or number of the dissimilarity column to use. By default, 
 #' the third column name of `dissimilarity` is used.
 #' 
-#' @param n_clust an `integer` or a `vector` of `integers` specifying the
+#' @param seed for the random number generator (NULL for random by default).
+#' 
+#' @param n_clust an `integer` or an `integer` vector specifying the
 #' requested number(s) of clusters
 #' 
 #' @param iter_max an `integer` specifying the maximum number of
-#' iterations for the kmeans method (see [stats::kmeans()][stats::kmeans])
+#' iterations for the kmeans method (see [kmeans][stats::kmeans])
 #' 
 #' @param nstart an `integer` specifying how many random sets of
 #' `n_clust` should be selected as starting points for the kmeans analysis
-#' (see [stats::kmeans()][stats::kmeans])
+#' (see [kmeans][stats::kmeans])
 #' 
-#' @param algorithm a `character string` specifying the algorithm to use for
-#' kmean (see [stats::kmeans()][stats::kmeans]). Available options are
+#' @param algorithm a `character` specifying the algorithm to use for
+#' kmean (see [kmeans][stats::kmeans]). Available options are
 #' Hartigan-Wong, Lloyd, Forgy and MacQueen.
+#' 
+#' @param algorithm_in_output a `boolean` indicating if the original output
+#' of [kmeans][stats::kmeans] should be returned in the output (`TRUE` by 
+#' default, see Value).
 #'
 #' @details
 #' This method partitions the data into k groups
@@ -32,29 +38,31 @@
 #' on dissimilarity/beta-diversity metrics, because these distances are not
 #' euclidean. Therefore, it requires first to transform the dissimilarity
 #' matrix with a Principal Coordinate Analysis (using the function
-#' [ape::pcoa()][ape::pcoa]), and then applying k-means on the coordinates
+#' [pcoa][ape::pcoa]), and then applying k-means on the coordinates
 #' of points in the PCoA. Because this makes an additional transformation of
 #' the initial matrix of dissimilarity, the partitioning around medoids method
-#' should be preferred ([nhclu_pam()])
-#'
-#' @seealso [nhclu_pam] 
-#'
+#' should be preferred ([nhclu_pam])
+#' 
 #' @return
 #' A `list` of class `bioregion.clusters` with five slots:
 #' \enumerate{
-#' \item{**name**: `character string` containing the name of the algorithm}
+#' \item{**name**: `character` containing the name of the algorithm}
 #' \item{**args**: `list` of input arguments as provided by the user}
 #' \item{**inputs**: `list` of characteristics of the clustering process}
 #' \item{**algorithm**: `list` of all objects associated with the
 #'  clustering procedure, such as original cluster objects}
 #' \item{**clusters**: `data.frame` containing the clustering results}}
+#' 
+#' In the `algorithm` slot, if `algorithm_in_output = TRUE`, users can
+#' find the output of
+#' [kmeans][stats::kmeans].
 #'
 #' @author
 #' Boris Leroy (\email{leroy.boris@gmail.com}),
 #' Pierre Denelle (\email{pierre.denelle@gmail.com}) and
 #' Maxime Lenormand (\email{maxime.lenormand@inrae.fr}) 
 #' 
-#' @seealso [cut_tree] 
+#' @seealso  [nhclu_pam]
 #' @examples
 #' comat <- matrix(sample(0:1000, size = 500, replace = TRUE, prob = 1/1:1001),
 #' 20, 25)
@@ -80,84 +88,69 @@
 
 nhclu_kmeans <- function(dissimilarity,
                          index = names(dissimilarity)[3],
-                         n_clust = NULL,
+                         seed = NULL,
+                         n_clust = c(1,2,3),
                          iter_max = 10,
                          nstart = 10,
-                         algorithm = "Hartigan-Wong"){
+                         algorithm = "Hartigan-Wong",
+                         algorithm_in_output = TRUE){
   
   # 1. Controls ---------------------------------------------------------------
-  if(inherits(dissimilarity, "bioregion.pairwise.metric")) {
-    if(attr(dissimilarity, "type") == "similarity") {
-      stop("dissimilarity seems to be a similarity object.
-         nhclu_kmeans() should be applied on dissimilarity, not similarities.
-         Use similarity_to_dissimilarity() before using nhclu_kmeans()")
-    }
-    if(is.numeric(index)) {
-      index <- names(dissimilarity)[index]
-    }
-    if(!(index %in% colnames(dissimilarity))) {
-      stop("Argument index should be one of the column names of dissimilarity")
-    }
-    
-  } else if(!any(inherits(dissimilarity, "bioregion.pairwise.metric"),
-                 inherits(dissimilarity, "dist"))) {
-    if(is.numeric(index)) {
-      index <- names(dissimilarity)[index]
-    }
-    if(is.null(index) || !(index %in% colnames(dissimilarity))) {
-      stop(
-        "dissimilarity is not a bioregion.pairwise.metric object, a dissimilarity
-        matrix (class dist) or a data.frame with at least 3 columns (site1,
-        site2, and your dissimilarity index)")
-    }
-  }
-  
-  if(!is.null(n_clust)){
-    if(is.numeric(n_clust)) {
-      if(any(!(n_clust %% 1 == 0))){
-        stop("n_clust must an integer or a vector of integers determining the
-             number of clusters.")
-      }
-      # Add test to see if n_clust is lower than the number of sites
-    } else {
-      stop("n_clust must an integer or a vector of integers determining the
-           number of clusters.")
-    }
-  } else{
-    stop("n_clust must an integer or a vector of integers determining the
-           number of clusters.")
-  }
-  
-  if(!inherits(dissimilarity, "dist")) {
+  controls(args = NULL, data = dissimilarity, type = "input_nhandhclu")
+  if(!inherits(dissimilarity, "dist")){
+    controls(args = NULL, data = dissimilarity, type = "input_dissimilarity")
+    controls(args = NULL, data = dissimilarity, 
+             type = "input_data_frame_nhandhclu")
+    controls(args = index, data = dissimilarity, type = "input_net_index")
+    net <- dissimilarity
+    net[, 3] <- net[, index]
+    net <- net[, 1:3]
+    controls(args = NULL, data = net, type = "input_net_index_value")
     dist.obj <- stats::as.dist(
-      net_to_mat(dissimilarity[, c(colnames(dissimilarity)[1:2], index)],
+      net_to_mat(net,
                  weight = TRUE, squared = TRUE, symmetrical = TRUE))
-    
   } else {
+    controls(args = NULL, data = dissimilarity, type = "input_dist")
     dist.obj <- dissimilarity
+    if(is.null(labels(dist.obj))){
+      attr(dist.obj, "Labels") <- paste0(1:attr(dist.obj, "Size"))
+      message("No labels detected, they have been assigned automatically.")
+    }
   }
   
+  if(!is.null(seed)){
+    controls(args = seed, data = NULL, type = "strict_positive_integer")
+  }
+  controls(args = n_clust, data = NULL, 
+           type = "strict_positive_integer_vector")
   controls(args = iter_max, data = NULL, type = "positive_integer")
   controls(args = nstart, data = NULL, type = "positive_integer")
-
-  if(!is.character(algorithm) || length(algorithm) != 1 ||
-     !(all(algorithm %in% c("Hartigan-Wong", "Lloyd", "Forgy", "MacQueen")))){
-    stop("algorithm is a character string indicating. Available options are
-         Hartigan-Wong, Lloyd, Forgy and MacQueen.")
+  controls(args = algorithm, data = NULL, type = "character")
+  if(!(algorithm %in% c("Hartigan-Wong", "Lloyd", "Forgy", "MacQueen"))){
+    stop("Please choose algorithm among the followings values:
+Hartigan-Wong, Lloyd, Forgy or MacQueen", call. = FALSE)
   }
+  controls(args = algorithm_in_output, data = NULL, type = "boolean")
 
   # 2. Function ---------------------------------------------------------------
-  outputs <- list(name = "kmeans")
+  outputs <- list(name = "nhclu_kmeans")
   
   outputs$args <- list(index = index,
+                       seed = seed,
                        n_clust = n_clust,
                        iter_max = iter_max,
                        nstart = nstart,
-                       algorithm = algorithm)
+                       algorithm = algorithm,
+                       algorithm_in_output = algorithm_in_output)
   
   outputs$inputs <- list(bipartite = FALSE,
                          weight = TRUE,
-                         pairwise_metric = TRUE,
+                         pairwise = TRUE,
+                         pairwise_metric = ifelse(!inherits(dissimilarity, 
+                                                            "dist"), 
+                                                  ifelse(is.numeric(index), 
+                                                         names(net)[3], index), 
+                                                  NA),
                          dissimilarity = TRUE,
                          nb_sites = attr(dist.obj, "Size"),
                          hierarchical = FALSE)
@@ -177,22 +170,33 @@ nhclu_kmeans <- function(dissimilarity,
   outputs$clustering_algorithms$pcoa <- ape::pcoa(dist.obj)
   
   # Performing the kmeans on the PCoA with all axes
-  outputs$algorithm$kmeans <- lapply(n_clust,
-                                     function(x)
-                                       stats::kmeans(dist.obj,
-                                                     centers = x,
-                                                     iter.max = iter_max,
-                                                     nstart = nstart,
-                                                     algorithm = algorithm))
-  
-  names(outputs$algorithm$kmeans) <- paste0("K_", n_clust)
+  if(is.null(seed)){
+    outputs$algorithm <- lapply(n_clust,
+                                function(x)
+                                  stats::kmeans(dist.obj,
+                                                centers = x,
+                                                iter.max = iter_max,
+                                                nstart = nstart,
+                                                algorithm = algorithm))
+  }else{
+    set.seed(seed)
+    outputs$algorithm <- lapply(n_clust,
+                                function(x)
+                                  stats::kmeans(dist.obj,
+                                                centers = x,
+                                                iter.max = iter_max,
+                                                nstart = nstart,
+                                                algorithm = algorithm))
+    rm(.Random.seed, envir=globalenv())
+  }
+  names(outputs$algorithm) <- paste0("K_", n_clust)
   
   outputs$clusters <- data.frame(
     outputs$clusters,
-    data.frame(lapply(names(outputs$algorithm$kmeans),
-                      function(x) outputs$algorithm$kmeans[[x]]$cluster)))
+    data.frame(lapply(names(outputs$algorithm),
+                      function(x) outputs$algorithm[[x]]$cluster)))
   
-  outputs$clusters <- knbclu(outputs$clusters, reorder = FALSE)
+  outputs$clusters <- knbclu(outputs$clusters, reorder = TRUE)
   
   outputs$cluster_info <- data.frame(
     partition_name = names(outputs$clusters)[2:length(outputs$clusters),
@@ -201,6 +205,11 @@ nhclu_kmeans <- function(dissimilarity,
                                      drop = FALSE],
                     2, function(x) length(unique(x))))
   
+  # Set algorithm in output
+  if (!algorithm_in_output) {
+    outputs$algorithm <- NA
+  }
+  
   class(outputs) <-  append("bioregion.clusters", class(outputs))
   
   return(outputs)
